/*
Copyright (c) 2015 Webtrends, Inc.
SharePoint 2013/SPO Plugin 
Version: 4.12.0.282
*/

(function () {
    var sp; // Store reference fo webtrends.sp.js file to borrow some fucntions for now

    var spsocial = function (dcs, plugin) {

        // Only run on newsfeeds currently - will update for My Site Blogs in V2
        // classic = #ms-microfeeddiv, modern = div.commentsWrapper_XXXXXXXX
        if (!sp.cfg.socialMenuClick || sp.cfg.legacySocial || (!document.querySelector("#ms-microfeeddiv") && !document.querySelector("div[class*='commentsWrapper']"))) {
            return;
        }

        /*********************************INITIALIZE*********************************/

        var impressions = typeof plugin.options.impressions ? plugin.options.impressions : false; // Impressions send one additional hit per visible post
        var trimCharLength = typeof plugin.options.textLength ? plugin.options.textLength : 50; // Trim post text length data
        var replyInProgress = false; // boolean to avoid double hits
        var postInProgress = false; // boolean to avoid double hits
        var feedTitle = ""; // Title of newsfeed filled in later
        var likes = false; // Flag for My Site newsfeed "Likes" tab which doesn't reload posts the same way as others.
        var version = sp.spVersion; // SharePoint version for On Enter logic (Enter vs Shift+Enter)

        // Determine the site type
        var siteType;
        switch (window.wt_sp_globals.spPageContextInfo.webTemplate) {
            case "1":
            case "64":
                siteType = "Site Newsfeed";
                break;
            case "9":
                if (window.wt_sp_globals.spPageContextInfo.webServerRelativeUrl &&
                    window.wt_sp_globals.spPageContextInfo.webServerRelativeUrl.indexOf("/my/") !== -1)
                    siteType = "My Site Blog";
                else siteType = "Blog";
                break;
            case "54":
                siteType = "My Site Newsfeed";
                break;
            default:
                siteType = "";
        }

        // Get the current visible post count on Page View, for impressions code
        var postCount = 0;
        dcs.addTransform(function (dcsObject, multiTrackObject) {
            try {
                if (!wt_sp_globals.modernView) {
                    // Clasic View
                    if (siteType === "Blog" || siteType === "My Site Blog") {
                        postCount = document.querySelectorAll("a.ms-accentText").length;
                    }
                    else if (siteType === "Site Newsfeed" || siteType === "My Site Newsfeed") {
                        postCount = document.querySelectorAll("div.ms-microfeed-thread").length;
                    }
                }
                else {
                    // Modern View
                    if (siteType === "Blog") {
                        postCount = ""; // TODO: Update when adding Blog Tracking
                    }
                    else if (siteType === "Site Newsfeed" ) {
                        postCount = document.querySelectorAll("div[data-automation-id='sp-comment-block-root']").length;
                    }
                }

            }
            catch (ex) { }
        }, 'collect');

        /*********************************FUNCTION DEFINITIONS*********************************/

        // Helper functions

        // Wrap cleanup function to add other values
        var finishCleanupAll = function (dcsObject, multiTrackObject) {
            dcsObject.WT.soc_content = "";
            dcsObject.WT.soc_feed_type = "";
            dcsObject.WT.soc_url = "";
            dcsObject.WT.soc_text = "";
            dcsObject.WT.soc_image = "";
            dcsObject.WT.soc_author = "";
            dcsObject.WT.soc_login = "";
            dcsObject.WT.soc_action = "";
            dcsObject.WT.soc_type = "";
            dcsObject.WT.soc_reply_text = "";
            dcsObject.WT.soc_reply_author = "";
            dcsObject.WT.soc_reply_login = "";
            dcsObject.WT.soc_reply_image = "";
            dcsObject.WT.soc_follow_author = "";
            dcsObject.WT.soc_follow_login = "";
            sp.finishCleanup(dcsObject, multiTrackObject);
        };

        // Extract the relevant ID portion of the element id.
        var getIDSubstring = function (el) {
            return el.id.substring(el.id.indexOf("_") + 1, el.id.length);
        };

        // Extract the relevant Post and Reply ID from the element id
        var getPostReplyID = function (id) {
            var result = { success: false };
            if (id) {
                var regex = /\.(\d+\.\d+)\./g;
                var reg = regex.exec(id);
                if (reg && reg[1].indexOf(".") >= 0) {
                    var ids = reg[1].split(".");
                    result.post = ids[0];
                    result.reply = ids[1];
                    result.success = true;
                }
            }
            return result;
        };

        // Get the post container element
        var getPostElement = function (el) {
            var container = null;
            if (!wt_sp_globals.modernView) {
                // Classic View
                if (el) {
                    // An existing post, crawl up the DOM to find the parent element.
                    container = sp.getParentByClassNameExact(el, "ms-microfeed-thread") || // Other
                        sp.getParentByClassNameExact(window.wt_sp_globals.tempElement, "ms-microfeed-thread ms-microfeed-greyout"); // Delete
                }
                else {
                    // A brand new post, just grab the first one on the page?
                    // This could be bad if they put two newsfeeds same page?
                    // What if they filter the mysite feed somehow?
                    container = document.querySelector("div.ms-microfeed-thread");
                }
                //	TODO: may need this - extra class for delete : ms-microfeed-thread ms-microfeed-greyout
                //  TODO: may need this - social follow user: "ms-microfeed-confirmationDiv ms-shadow ms-microfeed-rootConfDiv"
            }
            else {
                // Modern View
                if (el) {
                    var id = el.id.substring(el.id.indexOf("reply-") + 6, el.id.length);
                    container = document.querySelector("div[id^='sp-comment-" + id + "']");
                }
                else {
                    // A brand new post, just grab the first one on the page?
                    container = document.querySelector("div[id^='sp-comment-']:not(#sp-comment-input)");
                }
            }
            return container;
        };

        // Get the reply/comment container element
        var getReplyElement = function (el, newReply) {
            var container = null;
            if (el) {
                if (newReply) {
                    // New replies are added to the DOM, find it by checking for the highest list item id.
                    container = getLatestReply(el);
                }
                else {
                    // An existing reply, crawl up the DOM to find the parent element.
                    container = sp.getParentByClassNameExact(el, "ms-microfeed-replyBody") || // Other
                        sp.getParentByClassNameExact(window.wt_sp_globals.tempElement, "ms-microfeed-replyBody"); // Delete
                }
            }
            return container;
        };

        // Get the latest reply for a given post by looking for the highest list item id
        var getLatestReply = function (el) {
            var replies, maxId, replyId, i, ids, itemId;
            if (!wt_sp_globals.modernView) {
                // Classic View
                // Collect list of replies for this post based on the post ID
                replies = [].slice.call(document.querySelectorAll("div[threadDivID='" + getIDSubstring(el) + "']"));
                // Reverse the to hopefully place the newest first and avoid extra work
                replies.reverse();
                // Find the reply with the highest list id number
                maxId = 0;
                replyId = "";
                for (var i = 0; i < replies.length; i++) {
                    // Get the list item id from the id string
                    id = getIDSubstring(replies[i]);
                    ids = getPostReplyID(id);
                    if (ids.success) {
                        itemId = ids.reply || 0;
                        // Keep track of the highest item id
                        if (itemId > maxId) {
                            replyId = id;
                            maxId = itemId;
                        }
                    }
                }
                return document.querySelector("[id='ms-post_" + replyId + "']");
            }
            else {
                // Modern View
                // Collect list of replies and keep the highest number
                var par = sp.getParentByClassNameContains(el, "spComment_");
                if (par) replies = [].slice.call(par.querySelectorAll("div[data-automation-id='sp-comment-block-reply']"));
                else replies = [].slice.call(document.querySelectorAll("div[data-automation-id='sp-comment-block-reply']"));
                // Find the reply with the highest list id number
                maxId = 0;
                replyId = "";
                for (var i = 0; i < replies.length; i++) {
                    // Get the list item id from the id string
                    id = replies[i].id.replace("sp-comment-", "");
                    // Keep track of the highest item id
                    if (id > maxId) {
                        replyId = id;
                        maxId = id;
                    }
                }
                return document.querySelector("div[id='sp-comment-" + replyId + "']");
            }
        };

        // Initialize and get the post id, text, author, image, and create the proper URL
        var getPostData = function (el) {
            var res = {};
            res.id = "Post ID Not Found";
            res.content = "Post Text Not Found";
            res.author = "Post Author Not Found";
            res.image = false;
            var id, url;
            if (el) {
                if (!wt_sp_globals.modernView) {
                    // Classic View
                    // TODO: Need alternative method for id for My Site Blog Posts
                    id = el.hasAttribute("attachid") ? el.getAttribute("attachid") : getIDSubstring(el);
                    if (id) {
                        // Collect ID of post
                        res.id = id;
                        // Collect data from post element
                        res = getData(id, res);
                        // Append URL
                        url = getURL(el, id);
                        res.dcssip = url.dcssip;
                        res.dcsuri = url.dcsuri;
                        res.dcsqry = url.dcsqry;
                        res.dcsref = url.dcsref;
                    }
                }
                else {
                    // Modern View
                    id = el.hasAttribute("id") ? el.getAttribute("id") : "";
                    if (id) {
                        // Collect ID of post
                        res.id = id;
                        // Collect data from post element
                        res = getData(id, res);
                        // Append URL
                        url = getURL(el, id);
                        res.dcssip = url.dcssip;
                        res.dcsuri = url.dcsuri;
                        res.dcsqry = url.dcsqry;
                        res.dcsref = url.dcsref;
                    }
                }
            }
            return res;
        };

        // Initialize and get the reply/comment id, text, author, and image
        var getReplyData = function (el) {
            var res = {};
            res.id = "";
            res.content = "";
            res.author = "";
            res.login = "";
            res.image = false;
            if (el) {
                if (!wt_sp_globals.modernView)
                    res = getData(getIDSubstring(el), res);
                else 
                    res = getData(el.id, res);    
            }
            return res;
        };

        // Get the information from the post or reply container
        var getData = function (id, res) {
            var textEl, authorEl, loginEl, account, login, imgEl;
            res.id = id;
            if (!wt_sp_globals.modernView) {
                // Classic View
                // Collect post/comment text
                // TODO: Check up on this - If someone is @ in the text, it could be a problem for privacy options.
                // Possible fix is to Check for a.ms-subtleLink#ms-actorElement within ms-postbody, 
                // And the then use.replace on each name found within?
                textEl = document.querySelector("span[id^='ms-postBody_" + id + "']");
                if (textEl) {
                    var text = "";
                    // Encrypt @user names if desired
                    if (sp.cfg.obfuscate && textEl.querySelector("a.ms-subtleLink[href*='accountname=']")) {
                        var nodes = textEl.childNodes;
                        for (var i = 0; i < nodes.length; i++) {
                            if (!nodes[i].tagName) {
                                // Regular text
                                text += " " + sp.getElementText(nodes[i]);
                            }
                            else {
                                // @User Name
                                text += sp.dcsEncryption(sp.getElementText(nodes[i]));
                            }
                        }
                        res.content = trimText(text.replace(/^\s+|\s+$/g, ""));
                    }
                    else res.content = trimText(sp.getElementText(textEl));
                }

                // Collect post/comment author name
                authorEl = document.querySelector("[id^='ms-authorlabel_" + id + "']");
                if (authorEl) {
                    res.author = sp.getElementText(authorEl);
                }
                // Collect post/comment author login
                loginEl = document.querySelector("[id^='ms-authorlabel_" + id + "']");
                // SP2016 and SPO, the <a> is on a child element, not the ms-authorlabel_
                if (loginEl && loginEl.tagName !== "A") loginEl = loginEl.querySelector("a");
                if (loginEl) {
                    account = sp.getQueryParamValue("accountname", loginEl.getAttribute("href"));
                    if (account) {
                        login = decodeURIComponent(account);
                        // Online login users pipe delimited string
                        if (login.indexOf("|") >= 0) login = login.split("|").pop();
                        res.login = login;
                    }
                }
                // Determine if post/comment has image
                imgEl = document.querySelector("img[id^='ms-microfeed-attachmentImage_" + id + "']");
                if (imgEl) {
                    res.image = true;
                    if (!textEl) res.content = "(Image Only)";
                }
            }
            else {
                // Modern View
                // Collect post/comment text
                textEl = document.querySelector("div#" + id + " div[class^='spText_'] span[class^='commentRichText_']");
                if (textEl) {
                    var text = "";
                    // Encrypt @user names if desired
                    if (sp.cfg.obfuscate && textEl.querySelector("a[data-sp-mention-user-id]")) {
                        var nodes = textEl.childNodes;
                        for (var i = 0; i < nodes.length; i++) {
                            if (!nodes[i].tagName) {
                                // Regular text
                                text += " " + sp.getElementText(nodes[i]);
                            }
                            else {
                                // @User Name
                                text += " @" + sp.dcsEncryption(sp.getElementText(nodes[i]).replace(/^@/, ""));
                            }
                        }
                        res.content = trimText(text.replace(/^\s+|\s+$/g, ""));
                    }
                    else res.content = trimText(sp.getElementText(textEl));
                }
                // Collect post/comment author name
                authorEl = document.querySelector("div#" + id + " span[class^='spAuthor_']");
                if (authorEl) {
                    res.author = sp.getElementText(authorEl);
                }
                // Can't retrieve login from Modern Posts
                // Can't post images in Modern Posts
            }            
            return res;
        };

        /* TODO: Add with blog tracking
        // Get the post id, text, and author for Blog actions on home page
        var getBlogPostParams = function (elem) {
            var res = {};
            res.content = "Post Text Not Found";
            res.author = "Post Author Not Found";
            if (elem) {
                // Collect post text
                var text = elem.parentNode.querySelector("div[class^='ms-blog-postBody']>div>div>sp");
                if (text) res.content = sp.getElementText(text);
                // Collect post author
                var author = elem.parentNode.querySelector("a.ms-subtleLink");
                if (author) res.author = sp.getElementText(author);
                else res.author = wt_sp_globals.userName;
            }
            return res;
        };*/

        // Create the proper URL for a direct link to the post
        var getURL = function (elem, id) {
            var res = {};
            res.dcssip = window.location.hostname;
            res.dcsuri = (typeof asyncDeltaManager === "undefined") ? window.location.pathname : window.location.pathname + window.location.hash;
            res.dcsqry = window.location.search;
            res.dcsref = window.location.href;
            if (elem) {
                if (!wt_sp_globals.modernView) {
                    // Classic View
                    var uri = window.wt_sp_globals.spPageContextInfo.webServerRelativeUrl;
                    if (uri.lastIndexOf("/") === uri.length - 1) uri = uri.substring(0, uri.length - 1);
                    if (siteType === "My Site Newsfeed") {
                        res.dcssip = window.location.hostname;
                        res.dcsuri = uri + "/ThreadView.aspx";
                        res.dcsqry = "?ThreadID=" + id;
                    }
                    else if (siteType === "Site Newsfeed") {
                        res.dcssip = window.location.hostname;
                        res.dcsuri = uri + "/NewsFeed.aspx";
                        res.dcsqry = "?ThreadID=" + id;
                    }
                    /*    // TODO: - Not ready yet
                    else if (siteType === "Blog" || siteType === "My Site Blog") {
                        // Need to make new method to get appropriate ID - doesn't use element GUID like other's do
                        res.dcssip = window.location.hostname;
                        res.dcsuri = uri + "/Lists/Posts/Post.aspx";
                        res.dcsqry = id ? "?ID" + id : "";
                    }*/
                }
                else {
                    // Modern View
                    if (siteType === "Site Newsfeed") {
                        res.dcsuri += "#" + id;
                        res.dcsqry = "";
                    }
                    else if (siteType === "Blog") {
                        // Not ready yet
                    }
                }
                if (window.location.port) res.dcssip += ":" + window.location.port;
            }
            return res;
        };

        // Check if post or reply has an image
        var postHasImage = function (el) {
            if (!el) return false;
            var newPost = sp.getParentByIDContains(el, "ms-microblogdiv");
            if (newPost && newPost.querySelector("img.ms-microfeed-imageAttachmentPreview")) return true;
            else return false;
        };


        // Check if post or reply has an image
        var replyHasImage = function (el) {
            if (!el) return false;
            var reply = sp.getParentByIDContains(el, "ms-replyInputFocus");
            if (reply && reply.querySelector("img.ms-microfeed-imageAttachmentPreview")) return true;
            else return false;
        };

        // Check if the post text is empty, or too long, which will throw an error in SP
        var isValid = function (el, type) {
            if (!el) return false;
            var text;
            if (!wt_sp_globals.modernView) {
                // Classic View
                text = el.value ? el.value : "";
                // SP2013 limited to 512
                if (version === "2013") {
                    // Text not empty , less than 513, and not just whitespace
                    if (text.length > 0 && text.length < 513 && text.replace(/^\s+|\s+$/g, "") !== "") return true;
                    // No text, but an image
                    else if (type === "Post" && postHasImage(el) && text.length < 513) return true;
                    // No text, but an image
                    else if (type === "Reply" && replyHasImage(el) && text.length < 513) return true;
                }
                // SP2016 and SPO can do seemingly unlimited length
                else {
                    // Text not empty , and not just whitespace
                    if (text.length > 0 && text.replace(/^\s+|\s+$/g, "") !== "") return true;
                    // No text, but an image
                    else if (type === "Post" && postHasImage(el)) return true;
                    // No text, but an image
                    else if (type === "Reply" && replyHasImage(el)) return true;
                }
            }
            else {
                // Modern View
                text = sp.getElementText(el);
                // Text not empty, not too long, and not just whitespace
                if (text.length > 0 && text.length < 2001 && text.replace(/^\s+|\s+$/g, "") !== "") return true;
                // Can't attach images in Modern View
            }
            return false;
        };

        // Trim post text length to a specified amount for reporting
        var trimText = function (text) {
            // Trim post title length
            if (text && trimCharLength && text.length > trimCharLength) { // Only happens if the string is longer than the limit we defined
                text = text.substring(0, trimCharLength);  // Trims up to a certain number of characters
                text = text.substring(0, text.lastIndexOf(" ")) + "..."; // Trims on the last index of a space, then adds "..."
            }
            return text;
        };

        // Send the data to Webtrends
        var sendData = function (post, reply, action, actionTarget, webpart, navigation) {
            if (!post || !action) return;
            if (likes && post.getAttribute("displayInMyLikesView") === "false") return;
            if (!webpart) webpart = "";
            if (!navigation) navigation = "";
            var dl = (action === "Impression") ? "100" : "SHP";
            var title = (action === "Impression") ? ("Post Impression") : ("Link: " + action);
            var postData = getPostData(post);
            var replyData = getReplyData(reply);
            var followAuthor = "";
            var followAuthorLogin = "";
            var a = action.toLowerCase();
            if (a.indexOf("follow") >= 0 && a.indexOf("follow up") === -1) {
                followAuthor = replyData.author ? replyData.author : postData.author;
                followAuthorLogin = replyData.login ? replyData.login : postData.login;
            }
            window.Webtrends.multiTrack({
                argsa: [
                    "DCS.dcssip", postData.dcssip,
                    "DCS.dcsuri", postData.dcsuri,
                    "DCS.dcsqry", postData.dcsqry,
                    "DCS.dcsref", postData.dcsref,
                    "WT.ti", title,
                    "WT.dl", dl,
                    "WT.soc_content", feedTitle,
                    "WT.soc_feed_type", siteType,
                    "WT.soc_url", window.location.protocol + "//" + postData.dcssip + postData.dcsuri + postData.dcsqry,
                    "WT.soc_text", postData.content,
                    "WT.soc_author", postData.author,
                    "WT.soc_login", postData.login,
                    "WT.soc_action", action,
                    "WT.soc_image", postData.image,
                    "WT.soc_type", actionTarget,
                    "WT.soc_reply_text", replyData.content,
                    "WT.soc_reply_image", replyData.image,
                    "WT.soc_reply_author", replyData.author,
                    "WT.soc_reply_login", replyData.login,
                    "WT.soc_follow_author", followAuthor,
                    "WT.soc_follow_login", followAuthorLogin,
                    "WT.shp_wpv", webpart,
                    "WT.nv", navigation
                ],
                finish: function (dcsObject, multiTrackObject) {
                    finishCleanupAll(dcsObject, multiTrackObject);
                }
            });
        };

        /* TODO: Add with blog tracking - or combine with sendData above
        // Send the data to Webtrends for Blog (Blog home page)
        var sendDataBlog = function (element, action, socType, webpart, navigation) {
            if (!element || !action) return;
            if (!webpart) webpart = "";
            if (!navigation) navigation = "";
            var dl = (action === "Impression") ? "100" : "SHP";
            var title = (action === "Impression") ? ("Post Impression") : ("Blog - " + action);
            var soc = getBlogPostParams(element);
            var url = sp.getURIArrFromHREF("javascript:");
            Webtrends.multiTrack({
                argsa: [
                    "DCS.dcssip", url.dcssip,
                    "DCS.dcsuri", url.dcsuri,
                    "DCS.dcsqry", url.dcsqry,
                    "DCS.dcsref", url.dcsref,
                    "WT.ti", title,
                    "WT.dl", dl,
                    "WT.soc_content", feedTitle,
                    "WT.soc_feed_type", siteType,
                    "WT.soc_url", window.location.protocol + "//" + url.dcssip + url.dcsuri + url.dcsqry,
                    "WT.soc_text", soc.content,
                    "WT.soc_author", soc.author,
                    "WT.soc_author_login", soc.login,
                    "WT.soc_action", action,
                    "WT.soc_type", socType,
                    "WT.shp_wpv", webpart,
                    "WT.nv", navigation
                ],
                finish: function (dcsObject, multiTrackObject) {
                    finishCleanupAll(dcsObject, multiTrackObject);
                }
            });
        };*/

        /* TODO: Add with blog tracking, or combine with sendData above
        // Transform blog data to Webtrends
        var blogDataTransform = function (action, author, title, url, socType, webpart, navigation) {
            if (!action) return;
            if (!webpart) webpart = "";
            if (!navigation) navigation = "";
            var dl = (action === "Impression") ? "100" : "SHP";
            Webtrends.multiTrack({
                argsa: [
                    "DCS.dcssip", url.dcssip,
                    "DCS.dcsuri", url.dcsuri,
                    "DCS.dcsqry", url.dcsqry,
                    "DCS.dcsref", url.dcsref,
                    "WT.ti", action + " - " + title,
                    "WT.dl", dl,
                    "WT.soc_content", feedTitle,
                    "WT.soc_feed_type", siteType,
                    "WT.soc_url", window.location.protocol + "//" + url.dcssip + url.dcsuri + url.dcsqry,
                    "WT.soc_text", title,
                    "WT.soc_author", author,
                    "WT.soc_author_login", soc.login,
                    "WT.soc_action", action,
                    "WT.soc_type", socType,
                    "WT.shp_wpv", webpart,
                    "WT.nv", navigation
                ],
                finish: function (dcsObject, multiTrackObject) {
                    finishCleanupAll(dcsObject, multiTrackObject);
                }
            });
        };*/

        // Impression functions

        // Triggers the sending of Posts for impressions after links that change entire list
        // My Site Newsfeed links - Following, Everyone, Mentions
        // Does not run on "SHOW MORE POSTS"
        var triggerImpressions = function (dcsObject, options) {
            var event = options.event;
            var iterations = 0;
            var postsViewChange = setInterval(function (dcsObject) {
                if (iterations > 20) clearInterval(postsViewChange); // Quit trying after 10 seconds
                iterations++;
                // Wait until page is finished processing
                if (document.querySelector("div.ms-microfeed-processingDiv.ms-visibilityHidden")) {
                    clearInterval(postsViewChange);
                    setTimeout(function () {
                        var posts = document.querySelectorAll("div.ms-microfeed-thread");
                        for (var i = 0; i < posts.length; i++) {
                            var post = posts[i];
                            var reply = null;
                            if (!post.querySelector("div.ms-microfeed-oneLiner,div.ms-microfeed-activityText")) { // Ignore "notice" posts
                                var navigation = dcsObject ? dcsObject.dcsNavigation(event, dcsObject.navigationtag) : null;
                                sendData(post, reply, "Impression", "Post", sp.getWebPartTitleOnClick(post), navigation);
                            }
                        }
                        postCount = posts.length;
                        likes = false;
                    }, 1500);
                }
            }, 500);
        };

        // Post and Reply functions 
        // Trigger a Post, Reply, or Delete event on "Enter" key
        var triggerOnEnter = function (e) {
            try {
                if (!e) e = window.event;
                if (e.keyCode === 13) {
                    var element = (e.target || e.srcElement);
                    var id = element.id;
                    if (!wt_sp_globals.modernView) {
                        // Classic View
                        // Delete just needs an Enter event
                        if (id.indexOf("ms-confConfirmButton") === 0) {
                            var ids = getPostReplyID(id);
                            if (ids.success && ids.post === ids.reply) triggerDelete(e, window.wt_sp_globals.tempElement, "Post");
                            else triggerDelete(e, window.wt_sp_globals.tempElement, "Reply");
                        }
                        // SP2013 can post/reply on enter.  Others needs Shift + Enter
                        else if ((version === "2013" && !e.shiftKey) || (version !== "2013" && e.shiftKey)) {
                            if (id.indexOf("ms-microbloginputbox") === 0) {
                                // Avoid double posts
                                if (postInProgress) return;
                                else if (isValid(element, "Post")) triggerNewPost(e, element, null);
                            }
                            else if (id.indexOf("ms-replyinput") === 0) {
                                // Avoid double posts
                                if (replyInProgress) return false;
                                else {
                                    var par = sp.getParentByClassNameContains(element, "ms-microfeed-replyArea");
                                    if (par && isValid(par.querySelector("textarea"), "Reply")) triggerNewReply(e, element, null);
                                }
                            }
                        }
                    }
                    else {
                        // Modern View
                        // Delete just needs and enter event
                        if (element.hasAttribute("data-automation-id") && element.getAttribute("data-automation-id") === "yesButton") {
                            // This is too slow, and the elements are removed from the DOM before a hit fires.
                            if (window.wt_sp_globals.tempElement2) {
                                var post = window.wt_sp_globals.tempElement2;
                                var reply;
                                if (post.getAttribute("data-automation-id") === "sp-comment-block-reply") {
                                    reply = post;
                                    post = sp.getParentByClassNameContains(post.parentNode, "spComment_");
                                }
                                sendData(post, reply, "Delete", reply ? "Reply" : "Post", sp.getWebPartTitleOnClick(element), null);
                            }
                        }
                        // Post/Reply needs Ctrl + Enter
                        if (e.ctrlKey) {
                            if (id === "sp-comment-input") {
                                // Avoid double posts
                                if (postInProgress) return;
                                else if (isValid(element, "Post")) triggerNewPost(e, element, null);
                            }
                            else if (id.indexOf("sp-comment-input-reply-") === 0) {
                                // Avoid double posts
                                if (replyInProgress) return false;
                                else {
                                    var par = sp.getParentByClassNameContains(element, "spComment_");
                                    if (par && isValid(element, "Reply")) triggerNewReply(e, element, null);
                                }
                            }
                        }

                    }
                }
            }
            catch (ex) {
                postInProgress = false;
            }
        };

        // Wait for a pending Post to finish, then send the data of the topmost post, and update the current post count
        var triggerNewPost = function (e, el, dcsObject) {
            try {
                if (!e || !el) return;
                postInProgress = true;
                var navigation = dcsObject ? dcsObject.dcsNavigation(e, dcsObject.navigationtag) : null;
                // classic = ms-microfeed-thread, modern = div[data-automation-id='sp-comment-block-root']
                var previousCount = document.querySelectorAll("div.ms-microfeed-thread,div[data-automation-id='sp-comment-block-root']").length;
                var count = 0;
                var waitForPost = setInterval((function () {
                    try {
                        // If there's a SharePoint error pop up, or it's gone over 10 seconds trying to post, just quit
                        if (document.querySelector("#ms-errorDialogDiv") || count > 40) {
                            clearInterval(waitForPost);
                            postInProgress = false;
                        }
                        else {
                            if (!wt_sp_globals.modernView) {
                                // Classic View
                                // Send a hit if "post pending" is gone, and the post count has gone up.
                                var elems = document.querySelectorAll("div.ms-microfeed-thread");
                                if ((elems.length > previousCount && !document.querySelector("#ms-post_pending"))) {
                                    clearInterval(waitForPost);
                                    var post = getPostElement(null);
                                    var reply = null;
                                    sendData(post, reply, "Post", "Post", sp.getWebPartTitleOnClick(el), navigation);
                                    postCount = document.querySelectorAll("div.ms-microfeed-thread").length;
                                    postInProgress = false;
                                }
                            }
                            else {
                                // Modern View
                                // Send a hit if "post pending" is gone, and the post count has gone up.
                                var elems = document.querySelectorAll("div[data-automation-id='sp-comment-block-root']");
                                if ((elems.length > previousCount && !document.querySelector("div[id*='sp-comment-optimistic']"))) {
                                    clearInterval(waitForPost);
                                    var post = getPostElement(null);
                                    var reply = null;
                                    sendData(post, reply, "Post", "Post", sp.getWebPartTitleOnClick(el), navigation);
                                    postCount = document.querySelectorAll("div[data-automation-id='sp-comment-block-root']").length;
                                    postInProgress = false;
                                }
                            }

                        }
                        count++;
                    }
                    catch (ex) {
                        clearInterval(waitForPost);
                        postInProgress = false;
                    }
                }), 250);
            }
            catch (ex) {
                postInProgress = false;
            }
        };

        // Reply event handler
        var triggerNewReply = function (e, el, dcsObject) {
            try {
                replyInProgress = true;
                var count = 0;
                var post, reply;
                var waitForReply = setInterval(function () {
                    if (!wt_sp_globals.modernView) {
                        // If there's a SharePoint error pop up, or it's gone over 10 seconds trying to post, just quit
                        if (document.querySelector("#ms-errorDialogDiv") || count > 40) {
                            clearInterval(waitForReply);
                            replyInProgress = false;
                        }
                        // Check for the error dialogue or still pending reply in SPO - check on SP2016 for this?
                        else if (!document.querySelector("#ms-content_pending")) {
                            clearInterval(waitForReply);
                            post = getPostElement(el);
                            reply = getReplyElement(el, true);
                            sendData(post, reply, "Reply", "Post", sp.getWebPartTitleOnClick(el), (dcsObject ? dcsObject.dcsNavigation(e, dcsObject.navigationtag) : null));
                            setTimeout(function () {
                                replyInProgress = false;
                            }, 500);
                        }
                    }
                    else {
                        // Modern View
                        // Check for pending reply
                        if (!document.querySelector("div[id*='sp-comment-optimistic']")) {
                            clearInterval(waitForReply);
                            post = getPostElement(el);
                            reply = getReplyElement(el, true);
                            sendData(post, reply, "Reply", "Post", sp.getWebPartTitleOnClick(el), (dcsObject ? dcsObject.dcsNavigation(e, dcsObject.navigationtag) : null));
                            setTimeout(function () {
                                replyInProgress = false;
                            }, 500);
                        }
                    }
                    count++;
                }, 250);
            }
            catch (ex) {
                replyInProgress = false;
            }
        };

        // Delete event handler
        var triggerDelete = function (e, el, type) {
            var post = getPostElement(el);
            var reply = getReplyElement(el, false);
            sendData(post, reply, "Delete", type, sp.getWebPartTitleOnClick(el), null);
        };

        /*********************************NEWSFEEDS*********************************/

        // Get the current Feed Name, Feed Type, and Author if avaliable, runs on Page Views
        dcs.addTransform(function (dcsObject, multiTrackObject) {
            try {
                if (!multiTrackObject.argsa) multiTrackObject.argsa = [];
                feedTitle = (sp.getWebPartTitleOnClick(document.querySelector("#ms-microfeeddiv")) || document.title);
                var url = "";
                var text = "";
                var image = false;
                var author = "";
                var login = "";
                var action = "Feed View";
                var type = "";
                if (sp.getQueryParamValue("ThreadID", window.location.search)) {
                    var postData = getPostData(document.querySelector(".ms-microfeed-thread"));
                    url = window.location.protocol + "//" + postData.dcssip + postData.dcsuri + postData.dcsqry;
                    text = postData.content;
                    image = postData.image;
                    author = postData.author;
                    login = postData.login;
                    action = "Post View";
                    type = "Post";
                }
                multiTrackObject.argsa.push(
                    "WT.soc_content", feedTitle,
                    "WT.soc_feed_type", siteType,
                    "WT.soc_url", url,
                    "WT.soc_text", text,
                    "WT.soc_image", image,
                    "WT.soc_author", author,
                    "WT.soc_login", login,
                    "WT.soc_action", action,
                    "WT.soc_type", type
                );
            }
            catch (ex) { }
        }, 'collect');

        /*********************************POST IMPRESSIONS*********************************/
        if (impressions) {
            // Initial Post Impressions on Page View - Triggered by Collect event, which should allow this to work on MDS pages as well.
            dcs.addTransform(function (dcsObject, multiTrackObject) {
                try {
                    if (!wt_sp_globals.modernView) {
                        // Classic VIew
                        // Check if this is a thread view and abort, since the Page View will send it as a "Post View"?
                        if (!sp.getQueryParamValue("ThreadID", window.location.search)) {
                            setTimeout(function () {
                                var posts = document.querySelectorAll("div.ms-microfeed-thread");
                                for (var i = 0; i < posts.length; i++) {
                                    var post = posts[i];
                                    var reply = null;
                                    if (!post.querySelector("div.ms-microfeed-oneLiner,div.ms-microfeed-activityText")) { // Ignore "notice" posts
                                        sendData(post, reply, "Impression", "Post", sp.getWebPartTitleOnClick(post), null);
                                    }
                                }
                            }, 250);
                        }
                    }
                    else {
                        // Modern View
                        setTimeout(function () {
                            var posts = document.querySelectorAll("div[data-automation-id='sp-comment-block-root']");
                            for (var i = 0; i < posts.length; i++) {
                                var post = posts[i];
                                var reply = null;
                                sendData(post, reply, "Impression", "Post", sp.getWebPartTitleOnClick(post), null);
                            }
                        }, 250);
                    }
                }
                catch (ex) { }
            }, 'collect');

            // Classic View: SHOW MORE POSTS link - triggers for loop to send the additional impressions excluding those already sent.
            dcs.addSelector("button#ms-MoreThreadsButtonLabel", {
                actionElems: { 'BUTTON': 1 },
                filter: function (dcsObject, options) {
                    var e = options.event;
                    if (dcsObject._isRightClick(e)) return true;
                    postCount = document.querySelectorAll("div.ms-microfeed-thread").length;
                    var iterations = 0;
                    var showMorePosts = setInterval(function () {
                        var posts = document.querySelectorAll("div.ms-microfeed-thread");
                        if (iterations > 20 || (posts.length > postCount && document.querySelector("div.ms-microfeed-processingDiv.ms-visibilityHidden"))) {
                            clearInterval(showMorePosts);
                            for (var i = postCount; i < posts.length; i++) { // Start loop at previous postCount to avoid sending same post twice
                                var post = posts[i];
                                var reply = null;
                                if (!post.querySelector("div.ms-microfeed-oneLiner,div.ms-microfeed-activityText")) { // Ignore "notice" posts
                                    // Check if viewing a likes page to avoid too many impressions from hidden data in the HTML
                                    if (sp.translateLanguage(sp.getElementText(document.querySelector("a.ms-pivotControl-surfacedOpt-selected"))) === "Likes") likes = true;
                                    sendData(post, reply, "Impression", "Post", sp.getWebPartTitleOnClick(post), null);
                                }
                            }
                            postCount = posts.length;
                            likes = false;
                        }
                        iterations++;
                    }, 500);
                    return true;
                }
            });

            // Modern View: Show more comments - triggers for loop to send the additional impressions excluding those already sent.
            dcs.addSelector("button.ms-Link[class*='spMore_']", {
                actionElems: { 'BUTTON': 1 },
                filter: function (dcsObject, options) {
                    var e = options.event;
                    if (dcsObject._isRightClick(e)) return true;
                    postCount = document.querySelectorAll("div[data-automation-id='sp-comment-block-root']").length;
                    var iterations = 0;
                    var showMorePosts = setInterval(function () {
                        var posts = document.querySelectorAll("div[data-automation-id='sp-comment-block-root']");
                        if (iterations > 20 || posts.length > postCount) {
                            clearInterval(showMorePosts);
                            setTimeout(function () {
                                for (var i = postCount; i < posts.length; i++) { // Start loop at previous postCount to avoid sending same post twice
                                    var post = posts[i];
                                    var reply = null;
                                    sendData(post, reply, "Impression", "Post", sp.getWebPartTitleOnClick(post), null);
                                }
                                postCount = posts.length;
                            }, 1000);
                        }
                        iterations++;

                    }, 500);
                    return true;
                }
            });

            // Following/Everyone/Mentions links on My Site Newsfeed which shows alternative posts
            dcs.addSelector("a.ms-pivotControl-surfacedOpt", {
                actionElems: { 'A': 1, 'DIV': 1 },
                filter: function (dcsObject, options) {
                    var element = options.element;
                    var event = options.event;
                    if (dcsObject._isRightClick(event)) return true;
                    if (sp.translateLanguage(sp.getElementText(element)) === "Likes") likes = true;
                    else likes = false;
                    triggerImpressions(dcsObject, options);
                    return true;
                }
            });

            // My Site Newsfeed ECB Menu options, overflow menu of links above (Following,Everyone,Mentions,Likes,Activities)
            dcs.addSelector("div#ms-microfeeddiv a.ms-pivotControl-overflowDot", {
                filter: function (dcsObject, options) {
                    if (dcsObject._isRightClick(options.e)) return true;
                    (setTimeout(function () {
                        var obj = dcsObject;
                        var elems = document.querySelectorAll("div#ms-microfeeddiv a.ms-core-menu-link");
                        for (var i = 0; i < elems.length; i++) {
                            var elem = elems[i];
                            if (!elem.wtBound) {
                                // What about IE8?
                                elem.addEventListener(sp.setBindingOption(), function (e) {
                                    var element = e.target || e.srcElement;
                                    if (sp.translateLanguage(sp.getElementText(element)) === "Likes") likes = true;
                                    else likes = false;
                                    triggerImpressions(obj, options);
                                }, false);
                                elem.wtBound = true;
                            }
                        }
                    }, 1000));
                    return true;
                }
            });
        }

        /*********************************POST INTERACTIONS*********************************/

        // New Post (Click)
        dcs.addSelector("button#checkPost,button#ms-postbutton,button.ms-Button.ms-Button--primary", {
            actionElems: { 'BUTTON': 1 },
            filter: function (dcsObject, options) {
                var element = options.element;
                var event = options.event;
                // Avoid right clicks, disabled links, or double clicks
                if (dcsObject._isRightClick(event) || element.disabled || postInProgress) return true;
                var par, textarea;
                if (!wt_sp_globals.modernView) {
                    // Classic
                    par = sp.getParentByIDContains(element, "ms-microblogdiv");
                    if (par) {
                        textarea = par.querySelector("textarea");
                        if (isValid(textarea, "Post")) {
                            triggerNewPost(event, textarea, dcsObject);
                        }
                    }
                }
                else {
                    // Modern
                    // Abort for non social buttons
                    if (!sp.getParentByClassNameContains(element, "spComments_")) return true;
                    par = sp.getParentByClassNameContains(element, "spReply_");
                    if (par) {
                        textarea = par.querySelector("div#sp-comment-input");
                        if (isValid(textarea, "Post")) {
                            triggerNewPost(event, textarea, dcsObject);
                        }
                    }
                }
                return true;
            }
        });

        // New Post/Reply/Delete (On Enter)
        if (window.addEventListener) {
            document.addEventListener("keydown", triggerOnEnter, true);
        }
        else {
            // TODO: Not working in MSIE8 ?
            document.attachEvent("onkeydown", triggerOnEnter);
        }

        // Classic View: Like/Unlike a Post (Click)
        dcs.addSelector('a[id^="ms-likebutton_"]', {
            filter: function (dcsObject, options) {
                var el = options.element;
                var e = options.event;
                if (dcsObject._isRightClick(event)) return true;
                var action = !el.getAttribute("liked") ? "Like" : "Unlike";
                var post = getPostElement(el);
                var reply = getReplyElement(el, false);
                if (reply) {
                    // Reply Like/Unlike
                    sendData(post, reply, action, "Reply", sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                }
                else {
                    // Post Like/Unlike
                    sendData(post, reply, action, "Post", sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                }
                return true;
            },
            finish: function (dcsObject, multiTrackObject) {
                finishCleanupAll(dcsObject, multiTrackObject);
            }
        });

        // Modern View: Like/Unlike a Post (Click)
        dcs.addSelector('button.ms-Button.ms-Button--action', {
            actionElems: {'BUTTON': 1},
            filter: function (dcsObject, options) {
                var el = options.element;
                var e = options.event;
                if (dcsObject._isRightClick(event)) return true;
                if (!el.querySelector("i[data-icon-name='Like']") && !el.querySelector("i[data-icon-name='LikeSolid']")) return true;
                var action = el.querySelector("i[data-icon-name='Like']") ? "Like" : "Unlike";
                var post = sp.getParentByClassNameContains(el, "spComment_");
                var reply;
                if (post) {
                    // If the parent container is a sp-comment-block-reply, go up one more container for the parent post
                    if (post.getAttribute("data-automation-id") === "sp-comment-block-reply") {
                        reply = post;
                        post = sp.getParentByClassNameContains(post.parentNode, "spComment_");
                    }
                    if (reply) {
                        // Reply Like/Unlike
                        sendData(post, reply, action, "Reply", sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                    }
                    else {
                        // Post Like/Unlike
                        sendData(post, reply, action, "Post", sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                    }
                }
                return true;
            },
            finish: function (dcsObject, multiTrackObject) {
                finishCleanupAll(dcsObject, multiTrackObject);
            }
        });

        // Social Follow User
        dcs.addSelector('a[id*="followlinkfori"],a[id*="FollowLinkFor"]', {
            filter: function (dcsObject, options) {
                if (dcsObject._isRightClick(options.event)) return true;
                var el = options.element;
                var e = options.event;
                var action;
                var text = sp.translateLanguage(sp.getElementText(el)).toLowerCase(); 
                if (text.indexOf("stop") === 0) action = "Stop Following User";
                else action = "Follow User";
                var post = getPostElement(el);
                var reply = getReplyElement(el, false);
                if (reply) {
                    sendData(post, reply, action, "Reply", sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                }
                else {
                    sendData(post, reply, action, "Post", sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                }
                return true;
            },
            finish: function (dcsObject, multiTrackObject) {
                finishCleanupAll(dcsObject, multiTrackObject);
            }
        });

        /* TODO: Add later
        // Social Follow Hash
        dcs.addSelector('a[id^="ms-followbutton_"][name^="FollowButtonFor"]', {
            filter: function (dcsObject, options) {
                if (dcsObject._isRightClick(options.event)) return true;
                var el = options.element;
                var e = options.event;
                var action;
                var text = sp.translateLanguage(sp.getElementText(el)).toLowerCase();
                if (text.indexOf("stop") === 0) action = "Stop Following Hashtag";
                else action = "Follow Hashtag";
                var hash = text.substring(text.indexOf("#"), text.length);  // Pass the hash to a new parameter somehow.
                var post = getPostElement(el);
                var reply = getReplyElement(el, false);
                if (reply) {
                    sendData(post, reply, action, "Reply", sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                }
                else {
                    sendData(post, reply, action, "Post", sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                }
                return true;
            },
            finish: function (dcsObject, multiTrackObject) {
                finishCleanupAll(dcsObject, multiTrackObject);
            }
        });*/

        // Reply to a Post (Click)
        dcs.addSelector('button[id^="checkReply"],button[id^="ms-postreplybutton_"],button.ms-Button.ms-Button--primary', {
            actionElems: { 'BUTTON': 1 },
            filter: function (dcsObject, options) {
                try {
                    var e = options.event;
                    var el = options.element;
                    // Avoid right clicks, disabled links, or double clicks
                    if (dcsObject._isRightClick(e) || el.disabled || replyInProgress) return true;
                    var par;
                    if (!wt_sp_globals.modernView) {
                        // Classic View
                        par = sp.getParentByClassNameContains(el, "ms-microfeed-replyArea");
                        if (par && isValid(par.querySelector("textarea"), "Reply")) triggerNewReply(e, el, dcsObject);
                    }
                    else {
                        // Modern
                        // Abort for non social buttons
                        if (!sp.getParentByClassNameContains(el, "spComments_")) return true;
                        par = sp.getParentByClassNameContains(el, "spReply_");
                        if (par) {
                            var textarea = par.querySelector("div[id^='sp-comment-input-reply-']");
                            if (isValid(textarea, "Reply")) {
                                triggerNewReply(e, textarea, dcsObject);
                            }
                        }
                    }
                }
                catch (err) {
                    replyInProgress = false;
                }
                return true;
            },
            finish: function (dcsObject, multiTrackObject) {
                finishCleanupAll(dcsObject, multiTrackObject);
            }
        });

        // Classic View: Delete button - Stores a reference to the delete button for triggerOnEnter to find the post/reply
        dcs.addSelector("button.ms-microfeed-deleteButton", {
            actionElems: { 'BUTTON': 1 },
            filter: function (dcsObject, options) {
                window.wt_sp_globals.tempElement = options.element;
                return true;
            }
        });

        // Classic View: Delete a reply or comment (Click)
        dcs.addSelector('button.ms-microfeed-confirmationDivButton[id^="ms-confConfirmButton_"]', {
            actionElems: { 'BUTTON': 1 },
            filter: function (dcsObject, options) {
                var el = options.element;
                var e = options.event;
                if (dcsObject._isRightClick(e)) return true;
                // Get data for deleted post/reply
                var post = getPostElement(el);
                var reply = getReplyElement(el, false);
                if (reply) {
                    sendData(post, reply, "Delete", "Reply", sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                }
                else {
                    sendData(post, reply, "Delete", "Post", sp.getWebPartTitleOnClick(el), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                }
                // Update post count for impressions
                var postCountAfterDeletion = setInterval(function () {
                    if (document.querySelector(".ms-microfeed-processingDiv.ms-visibilityHidden")) {
                        clearInterval(postCountAfterDeletion);
                        postCount = document.querySelectorAll("div.ms-microfeed-thread").length;
                    }
                }, 500);
                // Exit
                return true;
            },
            finish: function (dcsObject, multiTrackObject) {
                finishCleanupAll(dcsObject, multiTrackObject);
            }
        });

        // Modern View: Delete button - Stores a reference to the delete button for deletions to track the proper link
        dcs.addSelector("button.ms-Button.ms-Button--action.ms-Button--command[class*='deleteAction']", {
            actionElems: { 'BUTTON': 1 },
            filter: function (dcsObject, options) {
                // Store the reply or post temporalily
                // Use .tempElement2 because .tempElement is used by webtrends.sp.js to store the subsequent ECB link and overwrites this.
                window.wt_sp_globals.tempElement2 = sp.getParentByClassNameContains(options.element, "spComment_");
                return true;
            }
        });

        // Modern View: Delete a reply or comment (Click)
        // dcs.addSelector('button.ms-ContextualMenu-link[data-automation-id="comment-delete-button"]', {  
        // This works on the ecb Delete link, but the confirm is preferable - confirm button may not have access to reply or comment if it's deleted too fast
        dcs.addSelector('button.ms-Button.ms-Button--primary[data-automation-id="yesButton"]', {
            actionElems: { 'BUTTON': 1 },
            filter: function (dcsObject, options) {
                var el = options.element;
                var e = options.event;
                if (dcsObject._isRightClick(e)) return true;
                if (!window.wt_sp_globals.tempElement2) return true;
                var post = window.wt_sp_globals.tempElement2;
                var reply;
                if (post) {
                    // If the parent container is a sp-comment-block-reply, go up one more container for the parent post
                    if (post.getAttribute("data-automation-id") === "sp-comment-block-reply") {
                        reply = post;
                        post = sp.getParentByClassNameContains(post.parentNode, "spComment_");
                    }
                    if (reply) {
                        // Reply Like/Unlike
                        sendData(post, reply, "Delete", "Reply", sp.getWebPartTitleOnClick(post), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                    }
                    else {
                        // Post Like/Unlike
                        sendData(post, reply, "Delete", "Post", sp.getWebPartTitleOnClick(post), dcsObject.dcsNavigation(e, dcsObject.navigationtag));
                    }
                }
                // Update post count for impressions
                setTimeout(function () {
                    postCount = document.querySelectorAll("div[data-automation-id='sp-comment-block-root']").length;
                }, 500);
                delete window.wt_sp_globals.tempElement2;
                return true;
            },
            finish: function (dcsObject, multiTrackObject) {
                finishCleanupAll(dcsObject, multiTrackObject);
            }
        });

        // Site Newsfeed ECB Menu options
        dcs.addSelector("a.ms-core-menu-root[id*=microfeed-moreDropDownMenu]", {
            filter: function (dcsObject, options) {
                if (dcsObject._isRightClick(options.e)) return true;
                var waitForDropDown = (setInterval(function () {
                    var elems = document.querySelectorAll("a.ms-core-menu-link");
                    if (elems.length > 0) {
                        clearInterval(waitForDropDown);
                        for (var i = 0; i < elems.length; i++) {
                            var elem = elems[i];
                            if (!elem.wtBound) {
                                // What about IE8?
                                elem.addEventListener(sp.setBindingOption(), function (e) {
                                    var obj = dcsObject;
                                    var el = e.target || e.srcElement;
                                    var text = sp.translateLanguage(sp.getElementText(el));
                                    // Delete is handled with confirmation pop up
                                    if (text === "Delete") return false;
                                    var post = getPostElement(el);
                                    var reply = getReplyElement(el, false);
                                    if (reply) {
                                        sendData(post, reply, text, "Reply", sp.getWebPartTitleOnClick(el), obj.dcsNavigation(e, obj.navigationtag));
                                    }
                                    else {
                                        sendData(post, reply, text, "Post", sp.getWebPartTitleOnClick(el), obj.dcsNavigation(e, obj.navigationtag));
                                    }
                                }, false);
                                elem.wtBound = true;
                            }
                        }
                    }
                }, 250));
                return true;
            }
        });

        // My Site Newsfeed "MENTIONS" tab "Show conversation" link
        dcs.addSelector("a.ms-microfeed-activityPostLink", {
            filter: function (dcsObject, options) {
                if (dcsObject._isRightClick(options.e)) return true;
                var id = getIDSubstring(options.element);
                var waitForConversation = (setInterval(function () {
                    var el = document.querySelector("[id*='" + id + "']");
                    if (el) {
                        clearInterval(waitForConversation);
                        var par = sp.getParentByClassNameExact(el, "ms-microfeed-thread");
                        var post = getPostElement(par);
                        sendData(post, null, "Post View", "Post", sp.getWebPartTitleOnClick(par), null); // dcsNavigation fails on this event
                    }
                }, 250));
                return true;
            }
        });

        /*********************************BLOG INTERACTIONS*********************************/
        /* TODO: Add with blog tracking
        // TODO: Investigate foreign language impact
        // Like a Post
        dcs.addSelector('a[id^="likesElement-"]', {
            filter: function (dcsObject, options) {
                var element = options.element;
                var event = options.event;
                if (dcsObject._isRightClick(event) || element.innerHTML.indexOf("Unlike") >= 0) return true;

                sendDataBlog(event, sp.getParentByClassNameExact(element, "ms-blog-commandSpace"), null, "Like", "Post", sp.getWebPartTitleOnClick(element), dcsObject.dcsNavigation(event, dcsObject.navigationtag));
                return true;
            },
            finish: function (dcsObject, multiTrackObject) {
                finishCleanupAll(dcsObject, multiTrackObject);
            }
        });

        // Unlike a Post
        dcs.addSelector('a[id^="likesElement-"]', {
            filter: function (dcsObject, options) {
                var element = options.element;
                var event = options.event;
                if (dcsObject._isRightClick(event) || element.innerHTML.indexOf("Like") >= 0) return true;

                sendDataBlog(event, sp.getParentByClassNameExact(element, "ms-blog-commandSpace"), null, "Unlike", "Post", sp.getWebPartTitleOnClick(element), dcsObject.dcsNavigation(event, dcsObject.navigationtag));
                return true;
            },
            finish: function (dcsObject, multiTrackObject) {
                finishCleanupAll(dcsObject, multiTrackObject);
            }
        });

        // Email a link for directly click on Blog home page
        dcs.addSelector('a[title="Email a link"]', {
            filter: function (dcsObject, options) {
                var element = options.element;
                var event = options.event;
                if (dcsObject._isRightClick(event)) return true;

                sendDataBlog(event, sp.getParentByClassNameExact(element, "ms-blog-commandSpace"), null, "Email a link", "Post", sp.getWebPartTitleOnClick(element), dcsObject.dcsNavigation(event, dcsObject.navigationtag));
                return true;
            },
            finish: function (dcsObject, multiTrackObject) {
                finishCleanupAll(dcsObject, multiTrackObject);
            }
        });

        // Post Comment (Click Post Button)
        dcs.addSelector('button[id*="-CommentSubmit"]', {
            actionElems: { BUTTON: 1 },
            filter: function (dcsObject, options) {
                if (dcsObject._isRightClick(options.event)) return true;
                var url = sp.getURIArrFromHREF(window.location.href);
                var title = "";
                var elem = document.querySelector("a.ms-accentText");
                if (elem) title = sp.getElementText(elem);
                title = trimText(title);
                blogDataTransform("Post", window.wt_sp_globals.userName, title, url, "Post", sp.getWebPartTitleOnClick(options.element), dcsObject.dcsNavigation(options.event, dcsObject.navigationtag));
                return true;
            },
            finish: function (dcsObject, multiTrackObject) {
                finishCleanupAll(dcsObject, multiTrackObject);
            }
        });

        // Lists Edit Posts/Comments Confirmation Button on EditPost.aspx/EditComment.aspx (Click Save/Publish Button when edit)
        dcs.addSelector('input[id*="diidIOSaveItem"]', {
            actionElems: { INPUT: 1 },
            filter: function (dcsObject, options) {
                if (dcsObject._isRightClick(options.event)) return true;
                if (window.location.href.indexOf("/NewPost.aspx") >= 0) return false;
                var url = sp.getURIArrFromHREF(window.location.href);
                // Get title for parameter WT.soc_text
                var title = "";
                var listObj = window.wt_sp_globals.pluginObj.listItemDetailsClassicView();
                if (listObj) title = listObj.item;
                // If it is edit post page, the action is Publish
                if (window.location.href.indexOf("/EditPost.aspx") >= 0) {
                    blogDataTransform("Publish", window.wt_sp_globals.userName, title, url, "Post", sp.getWebPartTitleOnClick(options.element), dcsObject.dcsNavigation(options.event, dcsObject.navigationtag));
                }
                // If it is edit comment page, the action is Save
                else {
                    blogDataTransform("Save", window.wt_sp_globals.userName, title, url, "Post", sp.getWebPartTitleOnClick(options.element), dcsObject.dcsNavigation(options.event, dcsObject.navigationtag));
                }
                return true;
            },
            finish: function (dcsObject, multiTrackObject) {
                finishCleanupAll(dcsObject, multiTrackObject);
            }
        });

        // Lists New Posts on NewPost.aspx (Create a Post in Blog tools)
        dcs.addSelector('input[id*="diidIOSaveItem"]', {
            actionElems: { INPUT: 1 },
            filter: function (dcsObject, options) {
                if (dcsObject._isRightClick(options.event)) return true;
                if (window.location.href.indexOf("/EditPost.aspx") >= 0 || window.location.href.indexOf("/EditComment.aspx") >= 0) return false;
                var newPostURL = window.location.href;
                var event = options.event;
                var element = options.element;
                try {
                    var count = 0;
                    var navigation = dcsObject ? dcsObject.dcsNavigation(event, dcsObject.navigationtag) : null;
                    var waitForPost = setInterval((function () {
                        try {
                            // If there's a SharePoint error pop up, or it's gone over 20 seconds trying to post, just quit
                            if (document.querySelector("#ms-errorDialogDiv") || count > 20) {
                                clearInterval(waitForPost);
                                postInProgress = false;
                            }
                            else {
                                // Send a hit if url is updated, and the post count has gone up.
                                var currentURL = window.location.href;
                                if (currentURL.indexOf("ID") && currentURL !== newPostURL) {
                                    clearInterval(waitForPost);
                                    sendDataBlog(event, sp.getParentByClassNameExact(element, "ms-blog-commandSpace"), null, "Post", "Post", sp.getWebPartTitleOnClick(element), navigation);
                                    postCount = document.querySelectorAll("ul.ms-blog-postList>li").length;
                                    postInProgress = false;
                                }
                            }
                            count++;
                        }
                        catch (ex) {
                            clearInterval(waitForPost);
                        }
                    }), 1000);
                }
                catch (ex) {
                    postInProgress = false;
                }

                //sessionStorage.wtPost = window.location.href;
                return true;
            },
            finish: function (dcsObject, multiTrackObject) {
                finishCleanupAll(dcsObject, multiTrackObject);
            }
        });

        // Ribbon : Delete Posts on EditPost.aspx
        dcs.addSelector('a[id^="Ribbon.PostListForm.Edit.Actions.DeleteItem"]', {
            filter: function (dcsObject, options) {
                if (dcsObject._isRightClick(options.event)) return true;
                var url = sp.getURIArrFromHREF(window.location.href);
                // Get title for parameter WT.soc_text
                var title = "";
                var listObj = window.wt_sp_globals.pluginObj.listItemDetailsClassicView();
                if (listObj) title = listObj.item;
                blogDataTransform("Delete Item", window.wt_sp_globals.userName, title, url, "Post", null, dcsObject.dcsNavigation(options.event, dcsObject.navigationtag));
                return true;
            },
            finish: function (dcsObject, multiTrackObject) {
                finishCleanupAll(dcsObject, multiTrackObject);
            }
        });

        // Ribbon : Lists Edit Posts/Comments Confirmation Button on EditPost.aspx/EditComment.aspx (Click Save/Publish Button when edit)
        dcs.addSelector('a[id^="Ribbon.PostListForm.Edit.Commit.Publish"],a[id^="Ribbon.ListForm.Edit.Commit.Publish"]', {
            filter: function (dcsObject, options) {
                if (dcsObject._isRightClick(options.event)) return true;
                if (window.location.href.indexOf("/NewPost.aspx") >= 0) return false;
                var url = sp.getURIArrFromHREF(window.location.href);
                // Get title for parameter WT.soc_text
                var title = "";
                var listObj = window.wt_sp_globals.pluginObj.listItemDetailsClassicView();
                if (listObj) title = listObj.item;
                // If it is edit post page, the action is Publish
                if (window.location.href.indexOf("/EditPost.aspx") >= 0) {
                    blogDataTransform("Publish", window.wt_sp_globals.userName, title, url, "Post", null, dcsObject.dcsNavigation(options.event, dcsObject.navigationtag));
                }
                // If it is edit comment page, the action is Save
                else {
                    blogDataTransform("Save", window.wt_sp_globals.userName, title, url, "Post", null, dcsObject.dcsNavigation(options.event, dcsObject.navigationtag));
                }
                return true;
            },
            finish: function (dcsObject, multiTrackObject) {
                finishCleanupAll(dcsObject, multiTrackObject);
            }
        });*/
    };

    // TODO: Consider incorporating this entire file into webtrends.sp.js - this section would be unnecessary
    // Register plugin, but wait for dom ready to make sure elements are present, and for sp plugin to be ready
    var register = function (tag, plugin) {
        // Boolean for dom readiness
        var domReady = false;
        if (document.readyState !== "loading") domReady = true;
        else if (document.addEventListener) document.addEventListener("DOMContentLoaded", function () { domReady = true; } , false);
        else document.attachEvent("DOMContentLoaded", function () { domReady = true; } );

        // Reference helper methods from webtrends.sp.js plugin (sp) for internal use, call worker function
        var waitForSpPlugin = setInterval(function () {
            if (domReady && window.wt_sp_globals.pluginObj) {
                clearInterval(waitForSpPlugin);
                sp = window.wt_sp_globals.pluginObj;
                spsocial(tag, plugin);                
            }
        }, 100);
    };
    window.Webtrends.registerPlugin("spsocial", register);
})();

